@@ -1,6 +1,7 @@
-// Copyright (C) 2020-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2020-2022 Daniel Mueller <deso@posteo.net>
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+use std::error::Error;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::fmt::Result as FmtResult;
@@ -21,6 +22,8 @@ impl Display for ErrorMessage {
   }
 }
 
+impl Error for ErrorMessage {}
+
 
 /// A macro used for defining the properties for a request to a
 /// particular HTTP endpoint.
@@ -41,7 +44,17 @@ macro_rules! Endpoint {
         /* 429 */ TOO_MANY_REQUESTS => RateLimitExceeded,
         $($(#[$err_docs])* $err_status => $variant,)*
       ],
+      ConversionErr => ::serde_json::Error,
       ApiErr => crate::endpoint::ErrorMessage,
+
+      fn parse(body: &[u8]) -> Result<Self::Output, Self::ConversionError> {
+        ::serde_json::from_slice::<Self::Output>(body)
+      }
+
+      fn parse_err(body: &[u8]) -> Result<Self::ApiError, Vec<u8>> {
+        ::serde_json::from_slice::<Self::ApiError>(body).map_err(|_| body.to_vec())
+      }
+
       $($defs)*
     }
   };
