@@ -1,30 +1,29 @@
-// Copyright (C) 2019-2021 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2019-2022 Daniel Mueller <deso@posteo.net>
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-use std::time::SystemTime;
+use chrono::serde::ts_milliseconds::deserialize as datetime_from_timestamp;
+use chrono::DateTime;
+use chrono::Utc;
 
 use futures::stream::unfold;
-use futures::FutureExt;
 use futures::Stream;
 use futures::StreamExt;
 
 use num_decimal::Num;
 
 use serde::Deserialize;
-use serde::Serialize;
-use serde_json::from_slice as from_json;
+use serde_json::from_slice as from_json_slice;
+use serde_json::from_str as from_json_str;
 use serde_json::Error as JsonError;
 
-use time_util::system_time_from_millis_in_new_york;
-use time_util::system_time_to_millis_in_new_york;
-
 use tracing::debug;
 use tracing::trace;
 
 use tungstenite::connect_async;
 
-use websocket_util::stream as do_stream;
 use websocket_util::tungstenite::Error as WebSocketError;
+use websocket_util::wrap::Message as WebSocketMessage;
+use websocket_util::wrap::Wrapper;
 
 use crate::api_info::ApiInfo;
 use crate::error::Error;
@@ -33,7 +32,7 @@ use crate::events::subscription::Subscription;
 
 
 /// A data point for a trade.
-#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[derive(Clone, Debug, Deserialize, PartialEq)]
 pub struct Trade {
   /// The stock's symbol.
   #[serde(rename = "sym")]
@@ -47,18 +46,14 @@ pub struct Trade {
   /// The number of shares traded.
   #[serde(rename = "s")]
   pub quantity: u64,
-  /// The trade's timestamp (in UNIX milliseconds).
-  #[serde(
-    rename = "t",
-    deserialize_with = "system_time_from_millis_in_new_york",
-    serialize_with = "system_time_to_millis_in_new_york",
-  )]
-  pub timestamp: SystemTime,
+  /// The trade's timestamp.
+  #[serde(rename = "t", deserialize_with = "datetime_from_timestamp")]
+  pub timestamp: DateTime<Utc>,
 }
 
 
 /// A quote for a stock.
-#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[derive(Clone, Debug, Deserialize, PartialEq)]
 pub struct Quote {
   /// The stock's symbol.
   #[serde(rename = "sym")]
@@ -81,19 +76,15 @@ pub struct Quote {
   /// The bid quantity
   #[serde(rename = "as")]
   pub ask_quantity: u64,
-  /// The quote's timestamp (in UNIX milliseconds).
-  #[serde(
-    rename = "t",
-    deserialize_with = "system_time_from_millis_in_new_york",
-    serialize_with = "system_time_to_millis_in_new_york",
-  )]
-  pub timestamp: SystemTime,
+  /// The quote's timestamp.
+  #[serde(rename = "t", deserialize_with = "datetime_from_timestamp")]
+  pub timestamp: DateTime<Utc>,
 }
 
 
 /// An aggregate for a stock.
 // TODO: Not all fields are hooked up.
-#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[derive(Clone, Debug, Deserialize, PartialEq)]
 pub struct Aggregate {
   /// The stock's symbol.
   #[serde(rename = "sym")]
@@ -116,20 +107,12 @@ pub struct Aggregate {
   /// The tick's low price.
   #[serde(rename = "l")]
   pub low_price: Num,
-  /// The tick's start timestamp (in UNIX milliseconds).
-  #[serde(
-    rename = "s",
-    deserialize_with = "system_time_from_millis_in_new_york",
-    serialize_with = "system_time_to_millis_in_new_york",
-  )]
-  pub start_timestamp: SystemTime,
-  /// The tick's end timestamp (in UNIX milliseconds).
-  #[serde(
-    rename = "e",
-    deserialize_with = "system_time_from_millis_in_new_york",
-    serialize_with = "system_time_to_millis_in_new_york",
-  )]
-  pub end_timestamp: SystemTime,
+  /// The tick's start timestamp.
+  #[serde(rename = "s", deserialize_with = "datetime_from_timestamp")]
+  pub start_timestamp: DateTime<Utc>,
+  /// The tick's end timestamp.
+  #[serde(rename = "e", deserialize_with = "datetime_from_timestamp")]
+  pub end_timestamp: DateTime<Utc>,
 }
 
 
@@ -199,7 +182,7 @@ pub(crate) type Messages = Vec<Message>;
 
 
 /// An enum representing the type of event we received from Polygon.
-#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[derive(Clone, Debug, Deserialize, PartialEq)]
 #[allow(clippy::large_enum_variant)]
 #[serde(tag = "ev")]
 pub enum Event {
@@ -347,9 +330,12 @@ where
   handshake(&mut stream, api_key, subscriptions).await?;
   debug!("subscription successful");
 
-  let stream = do_stream(stream)
-    .map(|stream| stream.map(|result| result.map(|data| from_json::<Messages>(&data))))
-    .await;
+  let stream = Wrapper::builder().build(stream).map(|result| {
+    result.map(|message| match message {
+      WebSocketMessage::Text(string) => from_json_str::<Messages>(&string),
+      WebSocketMessage::Binary(data) => from_json_slice::<Messages>(&data),
+    })
+  });
   let stream = Box::pin(stream);
   let stream = unfold(
     (false, (stream, Vec::new())),
@@ -375,12 +361,9 @@ mod tests {
   use futures::TryStreamExt;
 
   use serde_json::from_str as from_json;
-  use serde_json::to_string as to_json;
 
   use test_log::test;
 
-  use time_util::parse_system_time_from_str;
-
   use tungstenite::tungstenite::Message as WebSocketMessage;
 
   use url::Url;
@@ -427,15 +410,16 @@ mod tests {
     let addr = mock_server(f).await;
     let api_info = ApiInfo {
       api_url: Url::parse("http://example.com").unwrap(),
-      stream_url: Url::parse(&format!("ws://{}", addr.to_string())).unwrap(),
+      stream_url: Url::parse(&format!("ws://{}", addr)).unwrap(),
       api_key: API_KEY.to_string(),
     };
 
     stream(api_info, subscriptions).await
   }
 
+  /// Check that we can deserialize a `Trade`.
   #[test]
-  fn deserialize_serialize_trade() {
+  fn deserialize_trade() {
     let response = r#"{
       "ev": "T",
       "sym": "SPY",
@@ -447,23 +431,20 @@ mod tests {
       "t": 1583527402638,
       "z": 2
     }"#;
-    let trade = from_json::<Trade>(&response).unwrap();
+    let trade = from_json::<Trade>(response).unwrap();
     assert_eq!(trade.symbol, "SPY");
     assert_eq!(trade.exchange, 19);
     assert_eq!(trade.price, Num::new(29367, 100));
     assert_eq!(trade.quantity, 100);
     assert_eq!(
       trade.timestamp,
-      parse_system_time_from_str("2020-03-06T15:43:22.638Z").unwrap()
+      DateTime::parse_from_rfc3339("2020-03-06T15:43:22.638-05:00").unwrap()
     );
-
-    let json = to_json(&trade).unwrap();
-    let new = from_json::<Trade>(&json).unwrap();
-    assert_eq!(new, trade);
   }
 
+  /// Check that we can deserialize a `Quote`.
   #[test]
-  fn deserialize_serialize_quote() {
+  fn deserialize_quote() {
     let response = r#"{
       "ev": "Q",
       "sym": "SPY",
@@ -477,7 +458,7 @@ mod tests {
       "t": 1583527004684,
       "z": 2
     }"#;
-    let quote = from_json::<Quote>(&response).unwrap();
+    let quote = from_json::<Quote>(response).unwrap();
     assert_eq!(quote.symbol, "SPY");
     assert_eq!(quote.bid_exchange, 12);
     assert_eq!(quote.bid_price, Num::new(29431, 100));
@@ -487,16 +468,13 @@ mod tests {
     assert_eq!(quote.ask_quantity, 2);
     assert_eq!(
       quote.timestamp,
-      parse_system_time_from_str("2020-03-06T15:36:44.684Z").unwrap()
+      DateTime::parse_from_rfc3339("2020-03-06T15:36:44.684-05:00").unwrap()
     );
-
-    let json = to_json(&quote).unwrap();
-    let new = from_json::<Quote>(&json).unwrap();
-    assert_eq!(new, quote);
   }
 
+  /// Check that we can deserialize an `Aggregate`.
   #[test]
-  fn deserialize_serialize_aggregate() {
+  fn deserialize_aggregate() {
     let response = r#"{
       "ev": "A",
       "sym": "SPY",
@@ -513,7 +491,7 @@ mod tests {
       "e": 1583527402000
     }"#;
 
-    let aggregate = from_json::<Aggregate>(&response).unwrap();
+    let aggregate = from_json::<Aggregate>(response).unwrap();
     assert_eq!(aggregate.symbol, "SPY");
     assert_eq!(aggregate.volume, 2287);
     assert_eq!(
@@ -526,16 +504,12 @@ mod tests {
     assert_eq!(aggregate.low_price, Num::new(29368, 100));
     assert_eq!(
       aggregate.start_timestamp,
-      parse_system_time_from_str("2020-03-06T15:43:21Z").unwrap()
+      DateTime::parse_from_rfc3339("2020-03-06T15:43:21-05:00").unwrap()
     );
     assert_eq!(
       aggregate.end_timestamp,
-      parse_system_time_from_str("2020-03-06T15:43:22Z").unwrap()
+      DateTime::parse_from_rfc3339("2020-03-06T15:43:22-05:00").unwrap()
     );
-
-    let json = to_json(&aggregate).unwrap();
-    let new = from_json::<Aggregate>(&json).unwrap();
-    assert_eq!(new, aggregate);
   }
 
   #[test]
@@ -556,7 +530,7 @@ mod tests {
       "e": 1536036818784
     }"#;
 
-    let event = from_json::<Event>(&response).unwrap();
+    let event = from_json::<Event>(response).unwrap();
     match event {
       Event::MinuteAggregate(aggregate) => {
         assert_eq!(aggregate.symbol, "MSFT");
@@ -574,7 +548,7 @@ mod tests {
        "ap":59.89,"bs":28,"as":65,"t":1577724127207,"z":2}
     ]"#;
 
-    let messages = from_json::<Messages>(&response).unwrap();
+    let messages = from_json::<Messages>(response).unwrap();
     assert_eq!(messages.len(), 2);
     match &messages[0] {
       Message::Quote(Quote { symbol, .. }) if symbol == "XLE" => (),
